<TeXmacs|1.0.1.14>

<style|article>

<\body>
  <\expand|make-title>
    <title|<name|Yacas>: A do-it-yourself symbolic algebra environment>

    <author|Ayal Zwi Pinkus and Serge Winitzki>
  </expand>

  <\abstract>
    We describe the design and implementation of <name|Yacas>, a free
    computer algebra system currently under development. The system consists
    of a core interpreter and a library of scripts that implement symbolic
    algebra functionality. The interpreter provides a high-level weakly typed
    functional language designed for quick prototyping of computer algebra
    algorithms, but the language is suitable for all kinds of symbolic
    manipulation. It supports conditional term rewriting of symbolic
    expression trees, closures (pure functions) and delayed evaluation,
    dynamic creation of transformation rules, arbitrary-precision numerical
    calculations, and flexible user-defined syntax using infix notation. The
    library of scripts currently provides basic numerical and symbolic
    algebra functionality, such as polynomials and elementary functions,
    limits, derivatives and (limited) integration, solution of (simple)
    equations. The main advantages of <with|font series|bold|<with|font
    family|tt|Yacas>> are: free (GPL) software; a flexible and easy-to-use
    programming language with a comfortable and adjustable syntax;
    cross-platform portability and small resource requirements; and
    extensibility.
  </abstract>

  <\section>
    <value|hrule>

    Introduction
  </section>

  <with|font series|bold|<with|font family|tt|Yacas>> is a computer algebra
  system (CAS) which has been in development since the beginning of 1999. The
  goal was to make a small system that allows to easily prototype and
  research symbolic mathematics algorithms. A secondary future goal is to
  evolve <with|font series|bold|<with|font family|tt|Yacas>> into a
  full-blown general purpose CAS.

  <with|font series|bold|<with|font family|tt|Yacas>> is primarily intended
  to be a research tool for easy exploration and prototyping of algorithms of
  symbolic computation. The main advantage of <with|font
  series|bold|<with|font family|tt|Yacas>> is its rich and flexible scripting
  language. The language is closely related to LISP <with|font
  series|bold|<with|font family|tt|WH89>> but has a recursive descent infix
  grammar parser <with|font series|bold|<with|font family|tt|ASU86>>,
  includes expression transformation (term rewriting), and supports defining
  infix operators at run time similarly to Prolog <with|font
  series|bold|<with|font family|tt|B86>>.

  The <with|font series|bold|<with|font family|tt|Yacas>> language
  interpreter comes with a library of scripts that implement a set of
  computer algebra features. The <with|font series|bold|<with|font
  family|tt|Yacas>> script library is in active development and at the
  present stage does not offer the rich functionality of industrial-strength
  systems such as <with|font series|bold|<with|font family|tt|Mathematica>>
  or <with|font series|bold|<with|font family|tt|Maple>>. Extensive
  implementation of algorithms of symbolic computation is one of the future
  development goals.

  <with|font series|bold|<with|font family|tt|Yacas>> handles input and
  output in plain ASCII, either interactively or in batch mode. (A graphical
  interface is under development.) There is also an optional plugin mechanism
  whereby external libraries can be linked into the system to provide extra
  functionality.

  <label|c1s2>

  <\section>
    <value|hrule>

    Basic design
  </section>

  <with|font series|bold|<with|font family|tt|Yacas>> consists of a \0\0core
  engine" (kernel), which is an interpreter for the <with|font
  series|bold|<with|font family|tt|Yacas>> scripting language, and a library
  of script code.

  The <with|font series|bold|<with|font family|tt|Yacas>> engine has been
  implemented in a subset of C++ which is supported by almost all C++
  compilers. The design goals for <with|font series|bold|<with|font
  family|tt|Yacas>> core engine are: portability, self-containment (no
  dependence on extra libraries or packages), ease of implementing
  algorithms, code transparency, and flexibility. The <with|font
  series|bold|<with|font family|tt|Yacas>> system as a whole falls into the
  \0\0prototype/hacker" rather than into the \0\0axiom/algebraic" category,
  according to the terminology of Fateman <with|font series|bold|<with|font
  family|tt|F90>>. There are relatively few specific design decisions related
  to mathematics, but instead the emphasis is made on extensibility.

  The kernel offers sufficiently rich but basic functionality through a
  limited number of core functions. This core functionality includes
  substitutions and rewriting of symbolic expression trees, an infix syntax
  parser, and arbitrary precision numerics. The kernel does not contain any
  definitions of symbolic mathematical operations and tries to be as general
  and free as possible of predefined notions or policies in the domain of
  symbolic computation.

  The plugin inter-operability mechanism allows to extend the <with|font
  series|bold|<with|font family|tt|Yacas>> kernel or to use external
  libraries, e.g. GUI toolkits or implementations of special-purpose
  algorithms. A simple C++ API is provided for writing \0\0stubs'' that make
  external functions appear in <with|font series|bold|<with|font
  family|tt|Yacas>> as new core functions. Plugins are on the same footing as
  the <with|font series|bold|<with|font family|tt|Yacas>> kernel and can in
  principle manipulate all <with|font series|bold|<with|font
  family|tt|Yacas>> internal structures. Plugins can be compiled either
  statically or dynamically as shared libraries to be loaded at runtime from
  <with|font series|bold|<with|font family|tt|Yacas>> scripts.

  The script library contains declarations of transformation rules and of
  function syntax (prefix, infix etc.). The intention is that all symbolic
  manipulation algorithms, definitions of mathematical functions etc. should
  be held in the script library and not in the kernel. The only exception so
  far is for a very small number of mathematical or utility functions that
  are frequently used; they are compiled into the core for speed.

  <\subsection>
    <value|hrule>

    Portability
  </subsection>

  <with|font series|bold|<with|font family|tt|Yacas>> is designed to be as
  platform-independent as possible. All platform-specific parts have been
  clearly separated to facilitate porting. Even the standard C++ library is
  considered to be platform-specific, as there exist platforms without
  support for the standard C++ library (e.g. the EPOC32 platform).

  The primary development platform is GNU/Linux. Currently <with|font
  series|bold|<with|font family|tt|Yacas>> runs under various Unix variants,
  Windows environments, Psion organizers (EPOC32), Ipaq PDAs running the
  Linux kernel, BeOS, and Apple iMacs. Creating an executable for another
  platform (including embedded platforms) should not be difficult.

  <\subsection>
    <value|hrule>

    A self-contained system
  </subsection>

  <with|font series|bold|<with|font family|tt|Yacas>> should work as a
  standalone package, requiring minimum support from other operating system
  components. <with|font series|bold|<with|font family|tt|Yacas>> takes input
  and output in plain ASCII, either interactively or in batch mode. (An
  optional graphical interface is under development.) The system comes with
  its own (unoptimized) arbitrary precision arithmetic module but could be
  compiled to use another arbitrary precision arithmetic library; currently
  linking to <with|font series|bold|<with|font family|tt|gmp>> is
  experimentally supported. There is also an optional plugin mechanism
  whereby external libraries can be linked into the system to provide extra
  functionality.

  Self-containment is a requirement if the program is to be easy to port. A
  dependency on libraries that might not be available on other platforms
  would reduce portability. On the other hand, <with|font
  series|bold|<with|font family|tt|Yacas>> can be compiled with a complement
  of external libraries on "production" platforms.

  <\subsection>
    <value|hrule>

    Ease of use
  </subsection>

  <with|font series|bold|<with|font family|tt|Yacas>> is used mainly by
  executing programs written in the <with|font series|bold|<with|font
  family|tt|Yacas>> script language. A design goal is to create a high-level
  language that allows the user to conveniently express symbolic algorithms.
  A few lines of user code should go a long way.

  One major advantage of <with|font series|bold|<with|font family|tt|Yacas>>
  is the flexibility of its syntax. Although <with|font
  series|bold|<with|font family|tt|Yacas>> works internally as a LISP-style
  interpreter, all user interaction is through the <with|font
  series|bold|<with|font family|tt|Yacas>> script language which has a
  flexible infix grammar. Infix operators are defined by the user and may
  contain non-alphabetic characters such as "<with|font
  series|bold|<with|font family|tt|=>>" or "<with|font series|bold|<with|font
  family|tt|#>>". This means that the user interacts with <with|font
  series|bold|<with|font family|tt|Yacas>> using a comfortable and adjustable
  infix syntax, rather than a LISP-style syntax. The user can introduce such
  syntactic conventions as are most convenient for a given problem.

  For example, the <with|font series|bold|<with|font family|tt|Yacas>> script
  library defines infix operators "<with|font series|bold|<with|font
  family|tt|+>>", "<with|font series|bold|<with|font family|tt|*>>" and so on
  with conventional precedence, so that an algebraic expression can be
  entered in the familiar infix form such as\ 

  <with|color|blue|<\expand|quote>
    <expand|code*|(x+1)^2 - (y-2*z)/(y+3) + Sin(x*Pi/2)>
  </expand>>

  Once such infix operators are defined, it is possible to describe new
  transformation rules directly using the new syntax. This makes it easy to
  develop simplification or evaluation procedures adapted to a particular
  problem.

  Suppose the user needs to reorder expressions containing non-commutative
  creation and annihilation operators of quantum field theory. It takes about
  20 lines of <with|font series|bold|<with|font family|tt|Yacas>> script code
  to define an infix operation "<with|font series|bold|<with|font
  family|tt|**>>" to express non-commutative multiplication with the
  appropriate commutation relations and to automatically normal-order all
  expressions involving these symbols and other (commutative) factors. Once
  the operator <with|font series|bold|<with|font family|tt|**>> is defined
  (with precedence 4),

  <with|color|blue|<\expand|quote>
    <expand|code*|Infix("**", 4);>
  </expand>>

  the rules that express distributivity of the operation <with|font
  series|bold|<with|font family|tt|**>> with respect to addition may look
  like this:

  <\code>
    15 # (_x + _y) ** _z \<less\>-- x ** z + y ** z;

    15 # _z ** (_x + _y) \<less\>-- z ** x + z ** y;
  </code>

  Here, <with|font series|bold|<with|font family|tt|15 #>> is a specification
  of the rule precedence, <with|font series|bold|<with|font family|tt|_x>>
  denotes a pattern-matching variable <with|font series|bold|<with|font
  family|tt|x>> and the expression to the right of <with|font
  series|bold|<with|font family|tt|\<less\>-->> is to be substituted instead
  of a matched expression on the left hand side. Since all transformation
  rules are applied recursively, these two lines of code are enough for the
  <with|font series|bold|<with|font family|tt|Yacas>> engine to expand all
  brackets in any expression containing the infix operators <with|font
  series|bold|<with|font family|tt|**>> and <with|font series|bold|<with|font
  family|tt|+>>.

  Rule-based programming is not the only method that can be used in
  <with|font series|bold|<with|font family|tt|Yacas>> scripts; there are
  alternatives that may be more useful in some situations. For example, the
  familiar <with|font series|bold|<with|font family|tt|if>> / <with|font
  series|bold|<with|font family|tt|else>> constructs, <with|font
  series|bold|<with|font family|tt|For>>, <with|font series|bold|<with|font
  family|tt|ForEach>> loops are defined in the script library for the
  convenience of users.

  Standard patterns of procedural programming, such as subroutines that
  return values, with code blocks and temporary local variables, are also
  available. (A "subroutine" is implemented as a new "ground term" with a
  single rule defined for it.) Users may freely combine rules with C-like
  procedures or LISP-like list processing primitives such as <with|font
  series|bold|<with|font family|tt|Head()>>, <with|font
  series|bold|<with|font family|tt|Tail()>>.

  <\subsection>
    <value|hrule>

    Code clarity vs. speed
  </subsection>

  Speed is obviously an important factor. For <with|font
  series|bold|<with|font family|tt|Yacas>>, where a choice had to be made
  between speed and clarity of code, clarity was chosen. <with|font
  series|bold|<with|font family|tt|Yacas>> is mainly a prototyping system and
  its future maintainability is more important.

  This means that special-purpose systems designed for specific types of
  calculations, as well as heavily optimized industrial-strength computer
  algebra systems, will outperform <with|font series|bold|<with|font
  family|tt|Yacas>>. However, special-purpose or optimized external libraries
  can be dynamically linked into <with|font series|bold|<with|font
  family|tt|Yacas>> using the plugin mechanism.

  <\subsection>
    <value|hrule>

    Flexible, "policy-free" engine
  </subsection>

  The core engine of the <with|font series|bold|<with|font family|tt|Yacas>>
  system interprets the Yacas script language. The reason to implement yet
  another LISP-based custom language interpreter instead of taking an already
  existing one was to have full control over the system and to make it
  self-contained. While most of the features of the <with|font
  series|bold|<with|font family|tt|Yacas>> script language are "syntactic
  sugar" on top of a LISP interpreter, some features not commonly found in
  LISP systems were added.

  The script library contains declarations of transformation rules and of
  function syntax (prefix, infix etc.). The intention is that all symbolic
  manipulation algorithms, definitions of mathematical functions and so on
  should be held in the script library and not in the kernel. The only
  exception so far is for a very small number of mathematical or utility
  functions that are frequently used; they are compiled into the core for
  speed.

  For example, the mathematical operator "<with|font series|bold|<with|font
  family|tt|+>>" is an infix operator defined in the library scripts. To the
  kernel, this operator is on the same footing as any other function defined
  by the user and can be redefined. The <with|font series|bold|<with|font
  family|tt|Yacas>> kernel itself does not store any properties for this
  operator. Instead it relies entirely on the script library to provide
  transformation rules for manipulating expressions involving the operator
  "<with|font series|bold|<with|font family|tt|+>>". In this way, the kernel
  does not need to anticipate all possible meanings of the operator
  "<with|font series|bold|<with|font family|tt|+>>" that users might need in
  their calculations.

  This policy-free scheme means that <with|font series|bold|<with|font
  family|tt|Yacas>> is highly configurable through its scripting language. It
  is possible to create an entirely different symbolic manipulation engine
  based on the same C++ kernel, with different syntax and different naming
  conventions, by simply using another script library instead of the current
  library scripts. An example of the flexibility of the <with|font
  series|bold|<with|font family|tt|Yacas>> system is a sample script
  <with|font series|bold|<with|font family|tt|wordproblems.ys>> that comes
  with the distribution. It contains a set of rule definitions that make
  <with|font series|bold|<with|font family|tt|Yacas>> recognize simple
  English sentences, such as "Tom has 3 apples" or "Jane gave an apple to
  Tom", as valid <with|font series|bold|<with|font family|tt|Yacas>>
  expressions. <with|font series|bold|<with|font family|tt|Yacas>> can then
  "evaluate" these sentences to <with|font series|bold|<with|font
  family|tt|True>> or <with|font series|bold|<with|font family|tt|False>>
  according to the semantics of the current situation described in them.

  The "policy-free" concept extends to typing: strong typing is not required
  by the kernel, but can be easily enforced by the scripts if needed for a
  particular problem. The language offers features, but does not enforce
  their use. Here is an example of a policy implemented in the script
  library:

  <\code>
    61 # x_IsPositiveNumber ^ y_IsPositiveNumber\ 

    \ \ \ \ \ \ \<less\>-- MathPower(x,y);
  </code>

  By this rule, expressions of the form <with|font series|bold|<with|font
  family|tt|x^y>> (representing powers <with|font series|bold|x^y>) are
  evaluated and replaced by a number if and only if <with|font
  series|bold|<with|font family|tt|x>> and <with|font series|bold|<with|font
  family|tt|y>> are positive numerical constants. (The function <with|font
  series|bold|<with|font family|tt|MathPower>> is defined in the kernel.) If
  this simplification by default is not desirable, the user could erase this
  rule from the library and have a CAS without this feature.

  <\section>
    <value|hrule>

    The <with|font series|bold|<with|font family|tt|Yacas>> kernel
    functionality
  </section>

  <with|font series|bold|<with|font family|tt|Yacas>> script is a functional
  language based on various ideas that seemed useful for an implementation of
  CAS: list-based data structures, object properties, and functional
  programming (a la LISP); term rewriting [BN98] with pattern matching
  somewhat along the lines of Mathematica; user-defined infix operators a la
  PROLOG; delayed evaluation of expressions; and arbitrary-precision
  arithmetic. Garbage collection is implemented through reference counting.

  The kernel provides three basic data types: numbers, strings, and atoms,
  and two container types: list and static array (for speed). Atoms are
  implemented as strings that can be assigned values and evaluated. Boolean
  values are simply atoms <with|font series|bold|<with|font family|tt|True>>
  and <with|font series|bold|<with|font family|tt|False>>. Expression trees,
  association (hash) tables, stacks, and closures (pure functions) are all
  implemented using nested lists. In addition, more data types can be
  provided by plugins. Kernel primitives are available for
  arbitrary-precision arithmetic, string manipulation, array and list access
  and manipulation, for basic control flow, for assigning variables (atoms)
  and for defining rules for functions (atoms with a function syntax).

  The interpreter engine recursively evaluates expression trees according to
  user-defined transformation rules from the script library. Evaluation
  proceeds bottom-up, that is, for each function term, the arguments are
  evaluated first and then the function.

  A <with|font series|bold|<with|font family|tt|HoldArg()>> primitive is
  provided to not evaluate certain arguments of certain functions before
  passing them on as parameters to these functions. The <with|font
  series|bold|<with|font family|tt|Hold()>> and <with|font
  series|bold|<with|font family|tt|Eval()>> primitives, similarly to LISP's
  <with|font series|bold|<with|font family|tt|QUOTE>> and <with|font
  series|bold|<with|font family|tt|EVAL>>, can be used to stop the recursive
  application of rules at a certain point and obtain an unevaluated
  expression, or to initiate evaluation of an expression which was previously
  held unevaluated.

  When an expression can not be transformed any further, that is, when no
  more rules apply to it, the expression is returned unevaluated. For
  instance, a variable that is not assigned a value will return unevaluated.
  This is a desired behavior in a symbolic manipulation system.

  Rules are matched by a pattern expression which can contain <with|font
  shape|italic|pattern variables>, i.e. atoms marked by the "<with|font
  series|bold|<with|font family|tt|_>>" operator. During matching, each
  pattern variable atom becomes a local variable and is tentatively assigned
  the subexpression being matched. For example, the pattern <with|font
  series|bold|<with|font family|tt|_x + _y>> can match an expression
  <with|font series|bold|<with|font family|tt|a*x+b>> and then the pattern
  variable <with|font series|bold|<with|font family|tt|x>> will be assigned
  the value <with|font series|bold|<with|font family|tt|a*x>> (unevaluated)
  and the variable <with|font series|bold|<with|font family|tt|y>> will have
  the value <with|font series|bold|<with|font family|tt|b>>.

  This type of semantic matching has been frequently implemented before in
  various term rewriting systems (see, e.g., [C86]). However, the <with|font
  series|bold|<with|font family|tt|Yacas>> language offers its users an
  ability to create a much more flexible and powerful term rewriting system
  than one based on a fixed set of rules. Here are some of the features:

  First, transformation rules in <with|font series|bold|<with|font
  family|tt|Yacas>> have predicates that control whether a rule should be
  applied to an expression. Predicates can be any <with|font
  series|bold|<with|font family|tt|Yacas>> expressions that evaluate to the
  atoms <with|font series|bold|<with|font family|tt|True>> or <with|font
  series|bold|<with|font family|tt|False>> and are typically functions of
  pattern variables. A predicate could check the types or values of certain
  subexpressions of the matching context (see the <with|font
  series|bold|<with|font family|tt|_x ^ _y>> example in the previous
  subsection).

  Second, rules are assigned a precedence value (a positive integer) that
  controls the order of rules to be attempted. Thus <with|font
  series|bold|<with|font family|tt|Yacas>> provides somewhat better control
  over the automatic recursion than the pattern-matching system of <with|font
  series|bold|<with|font family|tt|Mathematica>> which does not allow for
  rule precedence. The interpreter will first apply the rule that matches the
  argument pattern, for which the predicate returns <with|font
  series|bold|<with|font family|tt|True>>, and which has the least
  precedence.

  Third, new rules can be defined dynamically as a side-effect of evaluation.
  This means that there is no predefined "ranking alphabet" of "ground terms"
  (in the terminology of [TATA99]), in other words, no fixed set of functions
  with predefined arities. It is also possible to define a "rule closure"
  that defines rules depending on its arguments, or to erase rules. Thus, a
  <with|font series|bold|<with|font family|tt|Yacas>> script library
  (although it is read-only) does not represent a fixed tree rewriting
  automaton. An implementation of machine learning is possible in <with|font
  series|bold|<with|font family|tt|Yacas>> (among other things). For example,
  when the module <with|font series|bold|<with|font
  family|tt|wordproblems.ys>> (mentioned in the previous subsection) "learns"
  from the user input that <with|font series|bold|<with|font
  family|tt|apple>> is a countable object, it defines a new postfix operator
  <with|font series|bold|<with|font family|tt|apples>> and a rule for its
  evaluation, so the expression <with|font series|bold|<with|font family|tt|3
  apples>> is later parsed as a function <with|font series|bold|<with|font
  family|tt|apples(3)>> and evaluated according to the rule.

  Fourth, <with|font series|bold|<with|font family|tt|Yacas>> expressions can
  be "tagged" (assigned a "property object" a la LISP) and tags can be
  checked by predicates in rules or used in the evaluation.

  Fifth, the scope of variables can be controlled. In addition to having its
  own local variables, a function can be allowed to access local variables of
  its calling environment (the <with|font series|bold|<with|font
  family|tt|UnFence()>> primitive). It is also possible to encapsulate a
  group of variables and functions into a "module", making some of them
  inaccessible from the outside (the <with|font series|bold|<with|font
  family|tt|LocalSymbols()>> primitive). The scoping of variables is a
  "policy decision", to be enforced by the script which defines the function.
  This flexibility is by design and allows to easily modify the behaviour of
  the interpreter, effectively changing the language as needed.

  <\section>
    <value|hrule>

    The <with|font series|bold|<with|font family|tt|Yacas>> scripting
    language
  </section>

  The <with|font series|bold|<with|font family|tt|Yacas>> interpreter is
  sufficiently powerful so that the functions <with|font
  series|bold|<with|font family|tt|While>>, <with|font series|bold|<with|font
  family|tt|For>>, <with|font series|bold|<with|font family|tt|ForEach>>,
  <with|font series|bold|<with|font family|tt|if>>, <with|font
  series|bold|<with|font family|tt|else>> etc., as well as the convenient
  shorthand "...<with|font series|bold|<with|font family|tt|\<less\>-->>..."
  for defining new rules, can be defined in the script library itself rather
  than in the kernel. This power is fully given to the user, since the
  library scrips are on the same footing as any user-defined code. Some
  library functions are intended mainly as tools available to a <with|font
  series|bold|<with|font family|tt|Yacas>> user to make algorithm
  implementation more comfortable. Below are some examples of the features
  provided by the <with|font series|bold|<with|font family|tt|Yacas>> script
  language.

  <with|font series|bold|<with|font family|tt|Yacas>> supports "function
  overloading": it allows a user to declare functions <with|font
  series|bold|<with|font family|tt|f(x)>> and <with|font
  series|bold|<with|font family|tt|f(x,y)>>, each having their own set of
  transformation rules. Of course, different rules can be defined for the
  same function name with the same number of arguments (arity) but with
  different argument patterns or different predicates.

  Simple transformations on expressions can be performed using rules. For
  instance, if we need to expand the natural logarithm in an expression, we
  could use the following rules:

  <\session|yacas|default>
    \;

    <\input|yacas] >
      log(_x * _y) \<less\>-- log(x) + log(y);
    </input>

    <\output>
      <\with|mode|math>
        <with|math font family|rm|True >
      </with>
    </output>

    <\input|yacas] >
      log(_x ^ _n) \<less\>-- n * log(x);
    </input>

    <\output>
      <\with|mode|math>
        <with|math font family|rm|True >
      </with>
    </output>

    \;
  </session>

  These two rules define a new symbolic function <with|font
  series|bold|<with|font family|tt|log>> which will not be evaluated but only
  transformed if one of these two rules are applicable. The symbol <with|font
  series|bold|<with|font family|tt|_>>, as before, indicates that the
  following atom is a pattern variable that matches subexpressions.

  After entering these two rules, the following interactive session is
  possible:

  <\session|yacas|default>
    <\input|yacas] >
      log(a*x^2)
    </input>

    <\output>
      <\with|mode|math>
        <with|math font family|rm|log ><left|(>a<right|)>+2<with|math font
        family|rm|log ><left|(>x<right|)>
      </with>
    </output>

    \;
  </session>

  Integration of the new function <with|font series|bold|<with|font
  family|tt|log>> can be defined by adding a rule for the <with|font
  series|bold|<with|font family|tt|AntiDeriv>> function atom,

  <\session|yacas|default>
    <\input|yacas] >
      AntiDeriv(_x,log(_x)) \<less\>-- x*log(x)-x;
    </input>

    <\output>
      <\with|mode|math>
        <with|math font family|rm|True >
      </with>
    </output>

    \;
  </session>

  Now <with|font series|bold|<with|font family|tt|Yacas>> can do integrations
  involving the newly defined <with|font series|bold|<with|font
  family|tt|log>> function, for example:

  <\session|yacas|default>
    <\input|yacas] >
      Integrate(x)log(a*x^n)
    </input>

    <\output>
      <\with|mode|math>
        <with|math font family|rm|log ><left|(>a<right|)>x+n<left|(>x<with|math
        font family|rm|log ><left|(>x<right|)>-x<right|)>
      </with>
    </output>

    <\input|yacas] >
      Integrate(x,B,C)log(a*x^n)
    </input>

    <\output>
      <\with|mode|math>
        <with|math font family|rm|log ><left|(>a<right|)>C+n<left|(>C<with|math
        font family|rm|log ><left|(>C<right|)>-C<right|)>-<left|(><with|math
        font family|rm|log ><left|(>a<right|)>B+n<left|(>B<with|math font
        family|rm|log ><left|(>B<right|)>-B<right|)><right|)>
      </with>
    </output>

    \;
  </session>

  Rules are applied when their associated patterns match and when their
  predicates return <with|font series|bold|<with|font family|tt|True>>. Rules
  also have precedence, an integer value to indicate which rules need to be
  applied first. Using these features, a recursive implementation of the
  integer factorial function may look like this in <with|font
  series|bold|<with|font family|tt|Yacas>> script,

  <\code>
    10 # Factorial(_n) _ (n=0) \<less\>-- 1;

    20 # Factorial(n_IsInteger) _ (n\<gtr\>0) \<less\>--

    \ \ n*Factorial(n-1);
  </code>

  Here the rules have precedence 10 and 20, so that the first rule will be
  tried first and the recursion will stop when <with|font series|bold|n=0> is
  reached.

  Rule-based programming can be freely combined with procedural programming
  when the latter is a more appropriate method. For example, here is a
  function that computes (<with|font series|bold|Mod(x^n,m)>) efficiently:

  <\code>
    powermod(x_IsPositiveInteger, n_IsPositiveInteger,

    \ \ \ \ m_IsPositiveInteger) \<less\>--

    [

    \ \ Local(result);

    \ \ result:=1;

    \ \ x:=Mod(x,m);

    \ \ While(n != 0)

    \ \ [

    \ \ \ \ if ((n&1) = 1)

    \t[

    \ \ \ \ \ \ result := Mod(result*x,m);

    \ \ \ \ ];

    \ \ \ \ x := Mod(x*x,m);

    \ \ \ \ n := n\<gtr\>\<gtr\>1;

    \ \ ];

    \ \ result;

    ];
  </code>

  Interaction with the function <with|font series|bold|<with|font
  family|tt|powermod(x,n,m)>> would then look like this:

  <\code>
    In\<gtr\> powermod(2,10,100)

    Out\<gtr\> 24;

    In\<gtr\> Mod(2^10,100)

    Out\<gtr\> 24;

    In\<gtr\> powermod(23234234,2342424234,232423424)

    Out\<gtr\> 210599936;
  </code>

  <label|c1s5>

  <\section>
    <value|hrule>

    Currently supported CAS features
  </section>

  <with|font series|bold|<with|font family|tt|Yacas>> consists of
  approximately 22000 lines of C++ code and 13000 lines of scripting code,
  with 170 functions defined in the C++ kernel and 600 functions defined in
  the scripting language. These numbers are deceptively small. The program is
  written in clean and simple style to keep it maintainable. Excessive
  optimization tends to bloat software and make it less readable.

  A base of mathematical capabilities has already been implemented in the
  script library (the primary sources of inspiration were the books [K98],
  [GG99] and [B86]). The script library is currently under active
  development. The following section demonstrates a few facilities already
  offered in the current system.

  Basic operations of elementary calculus have been implemented:

  <\session|yacas|defaul>
    \;

    <\input|yacas] >
      Limit(n,Infinity)(1+(1/n))^n
    </input>

    <\output>
      <\with|mode|math>
        exp <left|(>1<right|)>
      </with>
    </output>

    <\input|yacas] >
      Limit(h,0) (Sin(x+h)-Sin(x))/h
    </input>

    <\output>
      <\with|mode|math>
        cos x
      </with>
    </output>

    <\input|yacas] >
      Taylor(x,0,5)ArcSin(x)
    </input>

    <\output>
      <\with|mode|math>
        x+<frac|x<rsup|3>|6>+<frac|3x<rsup|5>|40>
      </with>
    </output>

    <\input|yacas] >
      InverseTaylor(x,0,5)Sin(x)
    </input>

    <\output>
      <\with|mode|math>
        <frac|3x<rsup|5>|40>+<frac|x<rsup|3>|6>+x
      </with>
    </output>

    <\input|yacas] >
      Integrate(x,a,b)Ln(x)+x
    </input>

    <\output>
      <\with|mode|math>
        b*ln b-b+<frac|b<rsup|2>|2>-<left|(>a*ln
        a-a+<frac|a<rsup|2>|2><right|)>
      </with>
    </output>

    <\input|yacas] >
      Integrate(x)1/(x^2-1)
    </input>

    <\output>
      <\with|mode|math>
        <frac|ln 2<left|(>x-1<right|)>|2>-<frac|ln 2<left|(>x+1<right|)>|2>
      </with>
    </output>

    <\input|yacas] >
      OdeSolve(y''==4*y)
    </input>

    <\output>
      <\with|mode|math>
        C<rsub|33>exp <left|(>-2x<right|)>+C<rsub|35>exp <left|(>2x<right|)>
      </with>
    </output>

    \;
  </session>

  Solving systems of equations has been implemented using a generalized
  Gaussian elimination scheme:

  <\session|yacas|default>
    \;

    <\input|yacas] >
      Solve( {x+y+z==6, 2*x+y+2*z==10, x+3*y+z==10}, {x,y,z}) [1]
    </input>

    <\output>
      <\with|mode|math>
        <left|(>4-z,2,z<right|)>
      </with>
    </output>

    \;
  </session>

  (The solution of this underdetermined system is returned as a vector, so
  <with|mode|math|x=4-z>, <with|mode|math|y=2>, and <with|mode|math|z>
  remains arbitrary.)

  A small theorem prover [B86] using a resolution principle is offered:

  <\session|yacas|default>
    <\input|yacas] >
      CanProve(P Or (Not P And Not Q))
    </input>

    <\output>
      <\with|mode|math>
        \<neg\>Q\<vee\>P
      </with>
    </output>

    <\input|yacas] >
      CanProve(a \<gtr\> 3 And a \<less\> 2)
    </input>

    <\output>
      <\with|mode|math>
        <with|math font family|rm|False >
      </with>
    </output>
  </session>

  Various exact and arbitrary-precision numerical algorithms have been
  implemented:

  <\session|yacas|default>
    <\input|yacas] >
      N(1/7,40); \ \ \ // evaluate to 40 digits
    </input>

    <\output>
      <\with|mode|math>
        0.1428571428571428571428571428571428571428
      </with>
    </output>

    <\input|yacas] >
      Decimal(1/7); \ \ \ // obtain decimal period
    </input>

    <\output>
      <\with|mode|math>
        <left|(>0,<left|(>1,4,2,8,5,7<right|)><right|)>
      </with>
    </output>

    <\input|yacas] >
      N(LnGamma(1.234+2.345*I)); \ \ \ // gamma-function
    </input>

    <\output>
      <\with|mode|math>
        0.70978922847119\<imath\>-2.1325569112
      </with>
    </output>

    \;
  </session>

  Various domain-specific expression simplifiers are available:

  <\session|yacas|default>
    <\input|yacas] >
      RadSimp(Sqrt(9+4*Sqrt(2)))
    </input>

    <\output>
      <\with|mode|math>
        <sqrt|8>+1
      </with>
    </output>

    <\input|yacas] >
      TrigSimpCombine(Sin(x)^2+Cos(x)^2)
    </input>

    <\output>
      <\with|mode|math>
        1
      </with>
    </output>

    <\input|yacas] >
      TrigSimpCombine(Cos(x/2)^2-Sin(x/2)^2)
    </input>

    <\output>
      <\with|mode|math>
        cos x
      </with>
    </output>

    <\input|yacas] >
      GcdReduce((x^2+2*x+1)/(x^2-1),x)
    </input>

    <\output>
      <\with|mode|math>
        <frac|x+1|x-1>
      </with>
    </output>

    \;
  </session>

  Univariate polynomials are supported in a dense representation, and
  multivariate polynomials in a sparse representation:

  <\session|yacas|default>
    <\input|yacas] >
      Factor(x^6+9*x^5+21*x^4-5*x^3-54*x^2-12*x+40)
    </input>

    <\output>
      <\with|mode|math>
        <left|(>x+2<right|)><rsup|3><left|(>x-1<right|)><rsup|2><left|(>x+5<right|)>
      </with>
    </output>

    <\input|yacas] >
      Apart(1/(x^2-x-2))
    </input>

    <\output>
      <\with|mode|math>
        <frac|1|3<left|(>x-2<right|)>>-<frac|1|3<left|(>x+1<right|)>>
      </with>
    </output>

    <\input|yacas] >
      Together(%)
    </input>

    <\output>
      <\with|mode|math>
        <frac|9|9x<rsup|2>-9x-18>
      </with>
    </output>

    <\input|yacas] >
      Simplify(%)
    </input>

    <\output>
      <\with|mode|math>
        <frac|1|x<rsup|2>-x-2>
      </with>
    </output>

    \;
  </session>

  Various "syntactic sugar" functions are defined to more easily enter
  expressions:

  <\session|yacas|default>
    <\input|yacas] >
      Ln(x*y) /: { Ln(_a*_b) \<less\>- Ln(a) + Ln(b) }
    </input>

    <\output>
      <\with|mode|math>
        ln x+ln y
      </with>
    </output>

    <\input|yacas] >
      Sum(x^(1 .. 5))
    </input>

    <\output>
      <\with|mode|math>
        <with|math font family|rm|Sum ><left|(><expand|tabular*|<tformat|<cwith|1|-1|1|1|cell
        halign|c>|<cwith|1|-1|2|2|cell halign|c>|<cwith|1|-1|3|3|cell
        halign|c>|<cwith|1|-1|4|4|cell halign|c>|<cwith|1|-1|5|5|cell
        halign|c>|<table|<row|<cell|x>|<cell|x<rsup|2>>|<cell|x<rsup|3>>|<cell|x<rsup|4>>|<cell|x<rsup|5>>>>>><right|)>
      </with>
    </output>

    <\input|yacas] >
      Select("IsPrime", 1 .. 15)
    </input>

    <\output>
      <\with|mode|math>
        <left|(>2,3,5,7,11,13<right|)>
      </with>
    </output>

    \;
  </session>

  Groebner bases [GG99] have been implemented:

  <\session|yacas|default>
    <\input|yacas] >
      Groebner({x*(y-1),y*(x-1)})
    </input>

    <\output>
      CommandLine(1) : Predicate doesn't evaluate to a boolean in pattern
    </output>

    \;
  </session>

  (From this it follows that <with|mode|math|x=y>, and <with|mode|math|x^2=x>
  so <with|mode|math|x> is <with|mode|math|0> or <with|mode|math|1>.)

  Symbolic inverses of matrices:

  <\session|yacas|default>
    <\input|yacas] >
      Inverse({{a,b},{c,d}})
    </input>

    <\output>
      <\with|mode|math>
        <left|(><expand|tabular*|<tformat|<cwith|1|-1|1|1|cell
        halign|c>|<cwith|1|-1|2|2|cell halign|c>|<table|<row|<cell|<frac|d|a*d-b*c>>|<cell|<frac|-b|a*d-b*c>>>|<row|<cell|<frac|-c|a*d-b*c>>|<cell|<frac|a|a*d-b*c>>>>>><right|)>
      </with>
    </output>

    \;
  </session>

  This list of features is not exhaustive.

  <inactive|<label|c1s6>>

  <\section>
    <value|hrule>

    Interface
  </section>

  Currently, <with|font series|bold|<with|font family|tt|Yacas>> is primarily
  a text-oriented application with interactive interface through the text
  console. Commands are entered and evaluated line by line; files containing
  longer code may be loaded and evaluated. A "notebook" interface under the
  GNU Emacs editor and <TeXmacs> are availables. There is also an
  experimental graphical interface (<with|font series|bold|<with|font
  family|tt|proteus>>) for Unix and Windows environments.

  Debugging facilities are implemented, allowing to trace execution of a
  function, trace application of a given rule pattern, or examine the stack
  when recursion did not terminate. An experimental debugger version of the
  <with|font series|bold|<with|font family|tt|Yacas>> executable that
  provides more detailed information can be compiled.

  <label|c1s7>

  <\section>
    <value|hrule>

    Documentation
  </section>

  The documentation for the <with|font series|bold|<with|font
  family|tt|Yacas>> is extensive and is actively updated, following the
  development of the system. Documentation currently consists of two tutorial
  guides (user's introduction and programmer's introduction), a collection of
  essays that describe some advanced features in more detail, and a full
  reference manual.

  <with|font series|bold|<with|font family|tt|Yacas>> currently comes with
  its own document formatting module that allows to maintain documentation in
  a special plain text format with a minimal markup. This text format is
  automatically converted to HTML, <LaTeX>, PostScript and PDF formats. The
  HTML version of the documentation is hyperlinked and is used as online help
  available from the <with|font series|bold|<with|font family|tt|Yacas>>
  prompt.

  <label|c1s8>

  <\section>
    <value|hrule>

    Future plans
  </section>

  The long-term goal for <with|font series|bold|<with|font family|tt|Yacas>>
  is to become an industrial-strength CAS and to remain a flexible research
  tool for easy prototyping of various methods of symbolic calculations.
  <with|font series|bold|<with|font family|tt|Yacas>> is meant to be a
  repository and a testbed for such algorithm prototypes.

  The plugin facility will be extended in the future, so that a rich set of
  extra additional libraries (especially free software libraries),
  system-specific as well as mathematics-oriented, should be loadable from
  the <with|font series|bold|<with|font family|tt|Yacas>> system. The issue
  of speed is also continuously being addressed.

  <label|c1s9>

  <\section>
    <value|hrule>

    References
  </section>

  [ASU86] A. Aho, R. Sethi and J. Ullman, <with|font shape|italic|Compilers
  (Principles, Techniques and Tools)>, Addison-Wesley, 1986.

  [B86] I. Bratko, <with|font shape|italic|Prolog (Programming for Artificial
  Intelligence)>, Addison-Wesley, 1986.

  [BN98] F. Baader and T. Nipkow, <with|font shape|italic|Term rewriting and
  all that>, Cambridge University Press, 1998.

  [C86] G. Cooperman, <with|font shape|italic|A semantic matcher for computer
  algebra>, in Proceedings of the symposium on symbolic and algebraic
  computation (1986), Waterloo, Ontario, Canada (ACM Press, NY).

  [F90] R. Fateman, <with|font shape|italic|On the design and construction of
  algebraic manipulation systems>, also published as: ACM Proceedings of the
  IAC-90, Tokyo, Japan.

  [GG99] J. von zur Gathen and J. Gerhard, <with|font shape|italic|Modern
  Computer Algebra>, Cambridge University Press, 1999.

  [K98] D. Knuth, <with|font shape|italic|The Art of Computer Programming
  (Volume 2, Seminumerical Algorithms)>, Addison-Wesley, 1998.

  [TATA99] H. Comon, M. Dauchet, R. Gilleron, F. Jacquemard, D. Lugiez, S.
  Tison, and M. Tommasi, <with|font shape|italic|Tree Automata Techniques and
  Applications>, 1999, online book: <hlink|http://www.grappa.univ-lille3.fr/tata|http://www.grappa.univ-lille3.fr/tata>

  [W96] S. Wolfram, <with|font shape|italic|The Mathematica book>, Wolfram
  Media, Champain, 1996.

  [WH89] P. Winston and B. Horn, <with|font shape|italic|LISP>,
  Addison-Wesley, 1989.
</body>

<\initial>
  <\collection>
    <associate|paragraph width|150mm>
    <associate|odd page margin|30mm>
    <associate|interparagraph space|5sep>
    <associate|page right margin|30mm>
    <associate|page top margin|30mm>
    <associate|reduction page right margin|25mm>
    <associate|reduction page bottom margin|15mm>
    <associate|first indentation|0>
    <associate|even page margin|30mm>
    <associate|reduction page left margin|25mm>
    <associate|page bottom margin|30mm>
    <associate|reduction page top margin|15mm>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|toc-10|<tuple|5|?>>
    <associate|toc-11|<tuple|6|?>>
    <associate|toc-12|<tuple|7|?>>
    <associate|toc-13|<tuple|8|?>>
    <associate|toc-14|<tuple|9|?>>
    <associate|toc-15|<tuple|9|?>>
    <associate|toc-16|<tuple|9|?>>
    <associate|toc-17|<tuple|9|?>>
    <associate|c1|<tuple|<uninit>|?>>
    <associate|c1s1|<tuple|<uninit>|?>>
    <associate|toc-1|<tuple|1|?>>
    <associate|c1s2|<tuple|1|?>>
    <associate|toc-2|<tuple|2|?>>
    <associate|toc-3|<tuple|2.1|?>>
    <associate|c1s3|<tuple|2.5|?>>
    <associate|c1s4|<tuple|3|?>>
    <associate|toc-4|<tuple|2.2|?>>
    <associate|toc-5|<tuple|2.3|?>>
    <associate|c1s5|<tuple|4|?>>
    <associate|toc-6|<tuple|2.4|?>>
    <associate|c1s6|<tuple|5|?>>
    <associate|toc-7|<tuple|2.5|?>>
    <associate|c1s7|<tuple|6|?>>
    <associate|toc-8|<tuple|3|?>>
    <associate|c1s8|<tuple|7|?>>
    <associate|toc-9|<tuple|4|?>>
    <associate|c1s9|<tuple|8|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font series|<quote|bold>|math font
      series|<quote|bold>|1<space|2spc><format|no first
      indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      Introduction><value|toc-dots><pageref|toc-1><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|math font
      series|<quote|bold>|2<space|2spc><format|no first
      indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      Basic design><value|toc-dots><pageref|toc-2><vspace|0.5fn>

      2.1<space|2spc><format|no first indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      Portability<value|toc-dots><pageref|toc-3>

      2.2<space|2spc><format|no first indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      A self-contained system<value|toc-dots><pageref|toc-4>

      2.3<space|2spc><format|no first indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      Ease of use<value|toc-dots><pageref|toc-5>

      2.4<space|2spc><format|no first indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      Code clarity vs. speed<value|toc-dots><pageref|toc-6>

      2.5<space|2spc><format|no first indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      Flexible, "policy-free" engine<value|toc-dots><pageref|toc-7>

      <vspace*|1fn><with|font series|<quote|bold>|math font
      series|<quote|bold>|3<space|2spc><format|no first
      indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      The <with|font series|<quote|bold>|<with|font family|<quote|tt>|Yacas>>
      kernel functionality><value|toc-dots><pageref|toc-8><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|math font
      series|<quote|bold>|4<space|2spc><format|no first
      indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      The <with|font series|<quote|bold>|<with|font family|<quote|tt>|Yacas>>
      scripting language><value|toc-dots><pageref|toc-9><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|math font
      series|<quote|bold>|5<space|2spc><format|no first
      indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      Currently supported CAS features><value|toc-dots><pageref|toc-10><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|math font
      series|<quote|bold>|6<space|2spc><format|no first
      indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      Interface><value|toc-dots><pageref|toc-11><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|math font
      series|<quote|bold>|7<space|2spc><format|no first
      indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      Documentation><value|toc-dots><pageref|toc-12><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|math font
      series|<quote|bold>|8<space|2spc><format|no first
      indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      Future plans><value|toc-dots><pageref|toc-13><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|math font
      series|<quote|bold>|9<space|2spc><format|no first
      indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      References><value|toc-dots><pageref|toc-14><vspace|0.5fn>
    </associate>
  </collection>
</auxiliary>
