<TeXmacs|1.0.1.14>

<style|tmdoc>

<\body>
  <label|c2>

  <chapter|Examples>

  \;

  This is a small tour of the capabilities <with|font series|bold|<with|font
  family|tt|Yacas>> currently offers. Note that this list of examples is far
  from complete. <with|font series|bold|<with|font family|tt|Yacas>> contains
  a few hundred commands, of which only a few are shown here.

  Additional example calculations including the results can be found here:

  <\itemize>
    <item>A selection of calculations from the <hlink|Wester
    benchmark|westerbench.html> .

    <item>Some additional <hlink|example calculations|mybench2.html> that
    <with|font series|bold|<with|font family|tt|Yacas>> can currently
    perform.
  </itemize>

  <\section>
    <value|hrule>

    Miscellaneous capabilities
  </section>

  <\session|yacas|default>
    <\input|yacas] >
      100!;
    </input>

    <\output>
      <\with|mode|math>
        93326215443944152681699238856266700490715968264381621468592963895217599993229915608941463976156518286253697920827223758251185210916864000000000000000000000000
      </with>
    </output>
  </session>

  Compute a large factorial using arbitrary precision integers.

  <\session|yacas|default>
    <\input|yacas] >
      ToBase(16,255);
    </input>

    <\output>
      <\with|mode|math>
        f*f
      </with>
    </output>
  </session>

  Convert to another number base.

  <\session|yacas|default>
    <\input|yacas] >
      Expand((1+x)^5);
    </input>

    <\output>
      <\with|mode|math>
        x<rsup|5>+5x<rsup|4>+10x<rsup|3>+10x<rsup|2>+5x+1
      </with>
    </output>
  </session>

  Expand the expression into a polynomial.

  <\session|yacas|default>
    <\input|yacas] >
      Apply("+",{2,3});
    </input>

    <\output>
      <\with|mode|math>
        5
      </with>
    </output>
  </session>

  Apply an operator to a list of arguments. This example evaluated to 5.

  <\session|yacas|default>
    <\input|yacas] >
      Apply({{x,y},x+y},{2,3});
    </input>

    <\output>
      <\with|mode|math>
        5
      </with>
    </output>
  </session>

  Apply a pure function to a list of arguments. This example would also
  evaluate to 5.

  <\session|yacas|default>
    <\input|yacas] >
      D(x)D(x) Sin(x);
    </input>

    <\output>
      <\with|mode|math>
        -sin x
      </with>
    </output>
  </session>

  Take derivatives of a function.

  <\session|yacas|default>
    <\input|yacas] >
      Solve(a+x*y==z,x);
    </input>

    <\output>
      <\with|mode|math>
        <frac|z-a|y>
      </with>
    </output>
  </session>

  Solve an equation for a variable.

  <\session|yacas|default>
    <\input|yacas] >
      Taylor(x,0,5) Sin(x);
    </input>

    <\output>
      <\with|mode|math>
        x-<frac|x<rsup|3>|6>+<frac|x<rsup|5>|120>
      </with>
    </output>
  </session>

  Calculate the Taylor series expansion of a function.

  <\session|yacas|default>
    <\input|yacas] >
      Limit(x,0) Sin(x)/x;
    </input>

    <\output>
      <\with|mode|math>
        1
      </with>
    </output>
  </session>

  Calculate the limit of a function as a variable approaches a value.

  <\session|yacas|default>
    <\input|yacas] >
      Newton(Sin(x),x,3,0.0001);
    </input>

    <\output>
      <\with|mode|math>
        3.1415926535
      </with>
    </output>
  </session>

  Use Newton's method for numerically finding a zero of a function.

  <\session|yacas|default>
    <\input|yacas] >
      DiagonalMatrix({a,b,c});
    </input>

    <\output>
      <\with|mode|math>
        <left|(><expand|tabular*|<tformat|<cwith|1|-1|1|1|cell
        halign|c>|<cwith|1|-1|2|2|cell halign|c>|<cwith|1|-1|3|3|cell
        halign|c>|<table|<row|<cell|a>|<cell|0>|<cell|0>>|<row|<cell|0>|<cell|b>|<cell|0>>|<row|<cell|0>|<cell|0>|<cell|c>>>>><right|)>
      </with>
    </output>
  </session>

  Create a matrix with the elements specified in the vector on the diagonal.

  <\session|yacas|default>
    <\input|yacas] >
      Integrate(x,a,b) x*Sin(x);
    </input>

    <\output>
      <\with|mode|math>
        sin b-b*cos b-<left|(>sin a-a*cos a<right|)>
      </with>
    </output>
  </session>

  Integrate a function over variable <verbatim|x>, from <verbatim|a> to
  <verbatim|b>.

  <\session|yacas|default>
    <\input|yacas] >
      Factors(x^2-1);
    </input>

    <\output>
      <\with|mode|math>
        <left|(><expand|tabular*|<tformat|<cwith|1|-1|1|1|cell
        halign|c>|<cwith|1|-1|2|2|cell halign|c>|<table|<row|<cell|x-1>|<cell|1>>|<row|<cell|x+1>|<cell|1>>>>><right|)>
      </with>
    </output>
  </session>

  Factorize a polynomial.

  <\session|yacas|default>
    <\input|yacas] >
      Apart(1/(x^2-1),x);
    </input>

    <\output>
      <\with|mode|math>
        <frac|1|2<left|(>x-1<right|)>>-<frac|1|2<left|(>x+1<right|)>>
      </with>
    </output>
  </session>

  Create a partial fraction expansion of a polynomial.

  <inactive|<label|c2s2>>

  <\section>
    <value|hrule>

    A longer calculation with plotting
  </section>

  Here is an example of a semi-real numerical calculation using <with|font
  series|bold|<with|font family|tt|Yacas>>. The task was to visualize a
  particular exact solution of an elliptical differential equation. The
  solution was found as an infinite series. We need to evaluate this infinite
  series numerically and plot it for particular values of the parameters.

  The function <with|mode|math|g(q,\<phi\>,\<chi\>)> is defined by

  <\equation>
    <with|mode|math|g(q,\<phi\>,\<chi\>)>=<frac|1|2*\<pi\>>*<frac|sin(q*\<phi\>)|sin(2*q*\<phi\>)>+<frac|1|\<pi\>>*<big|sum><rsup|\<infty\>><rsub|n=0>cos(n*\<chi\>)*<frac|sin(<sqrt|q<rsup|2>-n<rsup|2>>*\<phi\>)|sin(2*<sqrt|q<rsup|2>-n<rsup|2>>*\<phi\>)>
  </equation>

  The <name|Yacas> form of that formula is:

  <\session|yacas|default>
    <\input|yacas] >
      <\with|paragraph mode|center>
        g(q,phi,chi) := 1/(2*Pi)*Sin(q*phi)/Sin(2*q*phi)+1/Pi*Sum(n,0,Infinity,Cos(n*chi)*Sin(Sqrt(q^2-n^2)*phi)/Sin(2*Sqrt(q^2-n^2)*phi))
      </with>
    </input>

    <\output>
      <\with|mode|math>
        <with|math font family|rm|True >
      </with>
    </output>
  </session>

  wich is not very readable, but remember you can get a nice output inside
  the <TeXmacs> "notebook" interface, for complex funtions definitions,
  using:

  <\session|yacas|default>
    <\input|yacas] >
      TeXForm(g(q,phi,chi));
    </input>

    <\output>
      <\with|mode|math>
        <with|math font family|rm|mode|math|<frac|sin q\<phi\>|2\<pi\>sin
        2q\<phi\>>+<frac|<big|sum><rsub|n=0><rsup|\<infty\>>cos
        n\<chi\><frac|sin <sqrt|q<rsup|2>-n<rsup|2>>\<phi\>|sin
        2<sqrt|q<rsup|2>-n<rsup|2>>\<phi\>>|\<pi\>>>
      </with>
    </output>
  </session>

  So, like the <TeX> form is the same of the formula (1) (apart from cosmetic
  details), we are now pretty sure that we are not forgeting anything in our
  <name|Yacas> code definition of <with|mode|math|g(q,\<phi\>,\<chi\>)> \ and
  all parentesis were in their proper places. Go on then with our example.

  Here <with|mode|math|q>, <with|mode|math|\<phi\>> and
  <with|mode|math|\<chi\>> are numerical parameters of the problem. We would
  like to plot this series evaluated at fixed <with|mode|math|q> and
  <with|mode|math|\<phi\>> as function of <with|mode|math|\<chi\>> between 0
  and <with|mode|math|2*\<pi\>>.

  To solve this problem, we prepare a separate file with the following
  <with|font series|bold|<with|font family|tt|Yacas>> code:

  <\code>
    \ \ \ \ /* Auxiliary function */

    g1(n, q, phi, chi) := [

    \ \ \ \ Local(s);

    \ \ \ \ s := q^2-n^2;

    \ \ \ \ N(Cos(n*chi) * If(s=0,

    \ \ \ \ \ \ \ \ 1/2, \ \ \ /* Special case of s=0:

    \t\tavoid division by 0 */

    \ \ \ \ \ \ \ \ Sin(Sqrt(s)*phi)/Sin(2*Sqrt(s)*phi)

    \t\t \ /* now s != 0 */

    \ \ \ \ \ \ \ \ \ \ \ \ /* note that Sqrt(s) may

    \t\t\t \ be imaginary here */

    \ \ \ \ \ \ \ \ )

    \ \ \ \ );

    ];

    \ \ \ \ /* Main function */

    g(q, phi, chi) := [

    \ \ \ \ Local(M, n);

    \ \ \ \ M := 16;

    \t \ /* Exp(-M) will be the precision */

    \ \ \ \ \ \ /* Use N() to force numerical

    \t \ \ \ evaluation */

    \ \ \ \ N(1/2*Sin(q*phi)/Sin(2*q*phi)) +

    \ \ \ \ \ \ \ \ /* Estimate the necessary number

    \t\t \ of terms in the series */

    \ \ \ \ Sum(n, 1, N(1+Sqrt(q^2+M^2/phi^2)),

    \t \ g1(n, q, phi, chi)) ;

    ];

    \ \ \ \ /* Parameters */

    q:=3.5;

    phi:=2;

    \ \ \ \ /* Make a function for plotting:

    \t \ it must have only one argument */

    f(x) := g(q, phi, x);

    \ \ \ /* Plot from 0 to 2*Pi with 80 points */

    Plot2D(f(x), 0: 2*Pi);
  </code>

  Name this file "<with|font series|bold|<with|font family|tt|fun1>>" and
  execute this script by typing

  <\code>
    Load("fun1");
  </code>

  After this you should see a window with a plot.

  \;
</body>

<\initial>
  <\collection>
    <associate|paragraph width|150mm>
    <associate|odd page margin|5mm>
    <associate|page medium|automatic>
    <associate|page right margin|5mm>
    <associate|page top margin|5mm>
    <associate|reduction page right margin|25mm>
    <associate|reduction page bottom margin|15mm>
    <associate|even page margin|5mm>
    <associate|reduction page left margin|25mm>
    <associate|page height|768000unit>
    <associate|page bottom margin|5mm>
    <associate|reduction page top margin|15mm>
    <associate|page width|998400unit>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|c2|<tuple|<uninit>|?>>
    <associate|toc-1|<tuple|<uninit>|?>>
    <associate|c2s1|<tuple|<uninit>|?>>
    <associate|toc-2|<tuple|1|?>>
    <associate|c2s2|<tuple|1|?>>
    <associate|toc-3|<tuple|2|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|2fn><with|font series|<quote|bold>|math font
      series|<quote|bold>|font size|<quote|1.19>|Examples><value|toc-dots><pageref|toc-1><vspace|1fn>

      <vspace*|1fn><with|font series|<quote|bold>|math font
      series|<quote|bold>|1<space|2spc><format|no first
      indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      Miscellaneous capabilities><value|toc-dots><pageref|toc-2><vspace|0.5fn>

      <vspace*|1fn><with|font series|<quote|bold>|math font
      series|<quote|bold>|2<space|2spc><format|no first
      indentation><tformat|<tformat|<cwith|1|-1|1|-1|cell
      tborder|1ln>|<twith|table width|1par>|<cwith|1|-1|1|-1|cell
      vmode|exact>|<cwith|1|-1|1|-1|cell height|1ln>|<cwith|1|-1|1|-1|cell
      lsep|0fn>|<cwith|1|-1|1|-1|cell rsep|0fn>|<cwith|1|-1|1|-1|cell
      bsep|0fn>|<cwith|1|-1|1|-1|cell tsep|0fn>|<cwith|1|-1|1|-1|cell
      vcorrect|n>|<table|<row|<cell|<space|1fn|0ln|1ln>>>>>>

      A longer calculation with plotting><value|toc-dots><pageref|toc-3><vspace|0.5fn>
    </associate>
  </collection>
</auxiliary>
